@@ -355,7 +355,7 @@ spui_voice_add_modify_speaker_values_modify (SpeakerSettings *settings)
     gtk_range_set_value (GTK_RANGE (spui_speech_voice_add_modify.hs_voice_volume), settings->volume);
     gtk_range_set_value (GTK_RANGE (spui_speech_voice_add_modify.hs_voice_rate), 	settings->rate);
     gtk_range_set_value (GTK_RANGE (spui_speech_voice_add_modify.hs_voice_pitch), 	settings->pitch);
-    gtk_entry_set_text  (GTK_ENTRY (spui_speech_voice_add_modify.et_gnopernicus_voice), settings->gnopernicus_speaker);
+    gtk_entry_set_text  (GTK_ENTRY (spui_speech_voice_add_modify.et_gnopernicus_voice), _(settings->gnopernicus_speaker));
     gtk_entry_set_text  (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry), settings->voice_name_descr);
     gtk_entry_set_text  (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_driver)->entry), settings->driver_name_descr);
     
@@ -466,8 +466,9 @@ spui_voice_add_modify_driver_combo_changed (GtkWidget *widget,
 {
     const gchar  	*new_driver_name;
     const SpeechDriver  *driver;
+    const SpeechVoice	*sp_voice;
     SpeakerSettings 	*curr_setting = NULL;
-    GList		*list = NULL;
+    GList		*sp_list = NULL;
     
     if (!sp_curr_speaker_settings ||
 	!sp_changes_activable) 
@@ -483,30 +484,51 @@ spui_voice_add_modify_driver_combo_changed (GtkWidget *widget,
 	!strlen (new_driver_name))
 	return;
 
-	
+    /* get a new driver */	
     driver = spconf_driver_list_get_driver_from_name_descr (sp_speech_driver_list,
 						            new_driver_name);
 
+    /* verify if it is a valid driver */
     if (!driver)
 	return;
 	
     if (!strcmp (driver->driver_name, 
-		curr_setting->driver_name))
+		 curr_setting->driver_name))
 	return;
-	
-    list = spconf_voice_list_for_combo_get (sp_speech_driver_list, 
+
+    /* get list of speaker for driver name */	
+    sp_list = spconf_voice_list_for_combo_get (sp_speech_driver_list, 
 					    driver->driver_name_descr, 
 					    NULL);
 
-    spui_voice_add_modify_combo_list_set (list, spui_speech_voice_add_modify.cb_engine_voice); 
+    /* verify if the driver has a speaker */
+    if (!sp_list)
+	return;
 
-    spconf_list_for_combo_free (list);
+    /* set speaker list in combo box */
+    spui_voice_add_modify_combo_list_set (sp_list, spui_speech_voice_add_modify.cb_engine_voice); 
+    	
+    /* get speaker voice */
+    sp_voice  = spconf_driver_list_get_speech_voice (sp_speech_driver_list,
+						     driver->driver_name,
+						     (gchar*)sp_list->data);
 
+    /* free speaker list */
+    spconf_list_for_combo_free (sp_list);
+
+    /* free current driver and speaker values */
     g_free (curr_setting->driver_name);
     g_free (curr_setting->driver_name_descr);
+    g_free (curr_setting->voice_name);
+    g_free (curr_setting->voice_name_descr);
+    
+    /* set current driver and speaker values */
+    curr_setting->voice_name 	    = g_strdup (sp_voice->voice_name);
+    curr_setting->voice_name_descr  = g_strdup (sp_voice->voice_name_descr);
     curr_setting->driver_name 	    = g_strdup (driver->driver_name);
     curr_setting->driver_name_descr = g_strdup (driver->driver_name_descr);
     
+    /* save new settings if it is a new item*/
     if (!sp_new_item)
 	 spconf_speaker_settings_save (curr_setting);
 }
@@ -539,31 +561,14 @@ spui_voice_add_modify_speaker_combo_changed (GtkWidget *widget,
 	
     if (!strcmp (curr_setting->gnopernicus_speaker, NONE_ELEMENT))
 	return;
-/*
-    if (strcmp (curr_setting->voice_name_descr, voice_name) && 
-	strlen (voice_name) > 0 &&
-	strcmp (voice_name, NONE_ELEMENT) &&
-	strcmp (curr_setting->gnopernicus_speaker, NONE_ELEMENT))
-*/
-    {
-	g_free (curr_setting->voice_name);
-	g_free (curr_setting->voice_name_descr);
-	curr_setting->voice_name 	= g_strdup (sp_voice->voice_name);
-	curr_setting->voice_name_descr 	= g_strdup (sp_voice->voice_name_descr);
-/*
-	curr_setting->voice_name = g_strdup (
-				    spconf_driver_list_get_voice_name 
-					(sp_speech_driver_list,
-					 curr_setting->driver_name,
-			    		 voice_name
-					 )  );
 
-	curr_setting->voice_name_descr = g_strdup (voice_name);
-*/
-	if (!sp_new_item)
-	    spconf_speaker_settings_save (curr_setting);
-    }
-    
+    g_free (curr_setting->voice_name);
+    g_free (curr_setting->voice_name_descr);
+    curr_setting->voice_name 		= g_strdup (sp_voice->voice_name);
+    curr_setting->voice_name_descr 	= g_strdup (sp_voice->voice_name_descr);
+
+    if (!sp_new_item)
+        spconf_speaker_settings_save (curr_setting);
 }
 
 static void
@@ -593,7 +598,7 @@ spui_voice_add_modify_response (GtkDialog *dialog,
 	    sp_changes_activable = FALSE;
         break;
 	case GTK_RESPONSE_HELP: 
-	    gn_load_help ("gnopernicus-speech-settings");
+	    gn_load_help ("gnopernicus-speech-prefs");
 	    return;
 	break;
         default:
@@ -819,7 +824,7 @@ spui_voice_relative_response (GtkDialog *dialog,
 	case GTK_RESPONSE_OK: 
         break;
 	case GTK_RESPONSE_HELP: 
-	    gn_load_help ("gnopernicus-speech-settings");
+	    gn_load_help ("gnopernicus-speech-prefs");
 	    return;
 	break;
 	default:
@@ -1360,7 +1365,7 @@ spui_voice_absolute_response (GtkDialog *dialog,
 	case GTK_RESPONSE_OK: 
         break;
 	case GTK_RESPONSE_HELP: 
-	    gn_load_help ("gnopernicus-speech-settings");
+	    gn_load_help ("gnopernicus-speech-prefs");
 	    return;
 	break;
 	default:
@@ -1851,7 +1856,7 @@ spui_voice_settings_response (GtkDialog *dialog,
 	}
         break;
 	case GTK_RESPONSE_HELP: 
-	    gn_load_help ("gnopernicus-speech-settings");
+	    gn_load_help ("gnopernicus-speech-prefs");
 	break;
 	default:
 	    gtk_widget_hide ((GtkWidget*)dialog);
