@@ -1247,6 +1247,7 @@ ke_user_key_list_register_events (GSList *list)
 	    AccessibleKeySet 	*keyset = NULL;
 	    gchar		*keysym = NULL;
 
+	    modifiers |= SPI_KEYMASK_NUMLOCK;
 	    /* if no key go to next step */
 	    if (!key) 
 	    {
@@ -1637,7 +1638,8 @@ ke_user_key_list_free (GSList *list)
 {
     GSList *elem = NULL;    
 
-    sru_return_if_fail (list);
+    if (!list)
+	return;
 
     for (elem = list ; elem ; elem = elem->next)
 	g_free (elem->data);
