@@ -2049,8 +2049,9 @@ static gboolean
 src_surroundings_logic ()
 {
     SRObject *parent;
-    gchar *role;
-    gchar *message, *message1, *message2, *message3, *message4, *message5, *message6;
+    gchar *role, *name;
+    gchar *message, *on, *curr_obj_role, *curr_obj_name, *from, *parrent_role, *with, 
+          *no_items, *in, *window, *window_name, *of, *app_name;
     GSList *role_list, *cnt_list;
     GSList *crt, *crt2;
     
@@ -2072,44 +2073,60 @@ src_surroundings_logic ()
 	sro_release_reference (parent);
 	parent = tmp;
     }
+    
     if (!parent)
 	return FALSE;
     role_list = cnt_list = NULL;
     get_child_stop_role (parent, &role_list, &cnt_list);
-
     
-    message1 = NULL;
-    message2 = NULL;
     if (sro_get_role_name (src_crt_sro, &role, SR_INDEX_CONTAINER))
-    {
-	message1 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("on a"));
-    	message2 = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech (role));
+    {  
+	on = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("on"));
+    	curr_obj_role = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech (role));
 	SR_freeString (role);
     }
+    else
+    {
+	on = g_strdup ("");
+	curr_obj_role = g_strdup ("");
+    }
+    
+    if (sro_get_name (src_crt_sro, &name, SR_INDEX_CONTAINER))
+    {
+	curr_obj_name = src_xml_make_part ("TEXT", src_speech_get_voice ("name"), (gchar*)name);
+	SR_freeString (name);
+    }
+    else
+    {
+	curr_obj_name = g_strdup ("");
+    }
     
-    message3 = NULL;
-    message4 = NULL;
-    message5 = NULL;
     if (sro_get_role_name (parent, &role, SR_INDEX_CONTAINER))
     {
-	message3 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("from a"));
-	message4 = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech (role));
-	message5 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("with"));
+	from = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("from a"));
+	parrent_role = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech (role));
+	with = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("with"));
 	SR_freeString (role);
     }
+    else
+    {
+	from = g_strdup ("");
+	parrent_role = g_strdup ("");
+	with = g_strdup ("");
+    }
     
-    message6 = g_strdup ("");
+    no_items = g_strdup ("");
     for (crt = role_list, crt2 = cnt_list; crt; crt = crt->next, crt2 = crt2->next)
     {
     	gchar *tmp, *tmp2, *tmp3, *tmp4;
 	gchar cnt[10];
 	
-	tmp = message6;
+	tmp = no_items;
 	sprintf (cnt, "%d", GPOINTER_TO_INT (crt2->data));
 	tmp2 = src_xml_make_part ("TEXT", src_speech_get_voice ("childcount"), cnt);
 	tmp3 = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech ((gchar*) crt->data));
 	tmp4 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), ngettext ("item", "items", GPOINTER_TO_INT (crt2->data)));
-	message6 = g_strconcat (message6, tmp2, tmp3, tmp4, NULL);
+	no_items = g_strconcat (no_items, tmp2, tmp3, tmp4, NULL);
 	
 	g_free (tmp);
 	g_free (tmp2);
@@ -2117,11 +2134,47 @@ src_surroundings_logic ()
 	g_free (tmp4);
     }
 
+    if (sro_get_window_name (src_crt_sro, &role, &name, SR_INDEX_CONTAINER))
+    {
+	in = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("in a"));
+	window = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar*)role);
+	if (name)
+	    name = g_strconcat (_("named "), name, NULL);
+	else
+	    name = g_strdup (_("with no name"));    
+	window_name = src_xml_make_part ("TEXT", src_speech_get_voice ("name"), (gchar*)name);
+	SR_freeString (role);
+	SR_freeString (name);
+    }   
+    else
+    {
+	in = g_strdup ("");
+	window = g_strdup ("");
+	window_name = g_strdup ("");  
+    }
+	
+    if (sro_get_app_name (src_crt_sro, &name, SR_INDEX_CONTAINER))
+    {
+	of = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), _("of application"));
+	app_name = src_xml_make_part ("TEXT", src_speech_get_voice ("name"), (gchar*)name);
+	SR_freeString (name);
+    } 
+    else
+    {
+	of = g_strdup ("");
+	app_name = g_strdup ("");
+    }
+    
+    
     
     message = NULL;
-    if (message1 || message2 || message3 || message4 || message5 || message6)
-	message = g_strconcat (message1, message2, message3, 
-			    message4, message5, message6, NULL);
+    if (on || curr_obj_role || curr_obj_name || from || parrent_role 
+                 || with || no_items || in || window 
+		 || window_name || of || app_name)
+	message = g_strconcat (on, curr_obj_role, curr_obj_name, 
+			    from, parrent_role, with, no_items, 
+			    in, window, window_name, of, 
+			    app_name, NULL);
 
     if (message)
 	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
@@ -2130,12 +2183,18 @@ src_surroundings_logic ()
     fprintf (stderr, "\n%s", message); 
 #endif
     g_free (message);
-    g_free (message1);
-    g_free (message2);
-    g_free (message3);
-    g_free (message4);
-    g_free (message5);
-    g_free (message6);
+    g_free (on);
+    g_free (curr_obj_role);
+    g_free (curr_obj_name);
+    g_free (from);
+    g_free (parrent_role);
+    g_free (with);
+    g_free (no_items);
+    g_free (in);
+    g_free (window);
+    g_free (window_name);
+    g_free (of);
+    g_free (app_name);
 
     sro_release_reference (parent);
     g_slist_free (cnt_list);
@@ -3027,7 +3086,7 @@ static struct
 	{"unwatch all objects", 	src_unwatch_all	 		},
 
 	{"window hierarchy",		src_hierarchy_logic		},
-	{"window hierarchy flat",	src_hierarchy_flat		},
+	{"read whole window",	        src_hierarchy_flat		},
 	{"detailed informations",	src_detailed_informations	},
 	{"do default action",		src_do_default_action		},
 	{"window overview",		src_overview			},
